package hr.fer.tel.ruazosa.databaseexample;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.LinkedList;
import java.util.List;

public class ItemDatabase {

    //the index (key) column name
    public static final String KEY_ID = "ID";

    //names of columns
    public static final String KEY_NAME_COLUMN = "ITEM_NAME_COLUMN";
    public static final String KEY_VALUE_COLUMN = "ITEM_VALUE_COLUMN";
    public static final String KEY_ACCESSIBLE_COLUMN = "ITEM_ACCESSIBLE_COLUMN";

    private ItemDatabaseOpenHelper itemDatabaseOpenHelper;

    //constructor to instantiate the open helper
    public ItemDatabase(Context context) {
        itemDatabaseOpenHelper = new ItemDatabaseOpenHelper(context,
                ItemDatabaseOpenHelper.DATABASE_NAME, null,
                ItemDatabaseOpenHelper.DATABASE_VERSION);
    }

    public void closeDatabase() {
        itemDatabaseOpenHelper.close();
    }

    //add a new item
    public void addItem(Item item) {
        //create a new row of values to insert
        ContentValues newValues = new ContentValues();

        //assign values for each row
        newValues.put(KEY_NAME_COLUMN, item.getName());
        newValues.put(KEY_VALUE_COLUMN, item.getValue());
        newValues.put(KEY_ACCESSIBLE_COLUMN, item.getAccessible());

        //Insert the row into your table
        SQLiteDatabase db = itemDatabaseOpenHelper.getWritableDatabase();
        db.insert(ItemDatabaseOpenHelper.DATABASE_TABLE, null, newValues);
    }

    //get a single item
    public Item getItem(int id) {
        SQLiteDatabase db = itemDatabaseOpenHelper.getReadableDatabase();

        //defined arguments
        String[] result_columns = new String[]{KEY_NAME_COLUMN,
                KEY_VALUE_COLUMN, KEY_ACCESSIBLE_COLUMN};
        String where = KEY_ID + "=?";
        String[] where_args = new String[]{String.valueOf(id)};

        //undefined arguments
        String groupBy = null;
        String having = null;
        String order = null;

        Cursor cursor = db.query(ItemDatabaseOpenHelper.DATABASE_TABLE, result_columns,
                where, where_args, groupBy, having, order);
        if (cursor != null)
            cursor.moveToFirst();

        Item item = new Item(cursor.getString(0), cursor.getFloat(1),
                cursor.getInt(2));

        //return item
        return item;
    }

    //get all accessible items
    public List<Item> getAllItems() {
        List<Item> itemList = new LinkedList<Item>();

        //select all items
        String selectQuery = "SELECT  * FROM " + ItemDatabaseOpenHelper.DATABASE_TABLE;

        SQLiteDatabase db = itemDatabaseOpenHelper.getReadableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);

        //looping through all rows and adding to list
        if (cursor.moveToFirst()) {
            do {
                Item item = new Item(cursor.getString(1), cursor.getFloat(2),
                        cursor.getInt(3));
                //add item to list
                itemList.add(item);
            } while (cursor.moveToNext());
        }

        //return item list
        return itemList;
    }

    //clear all items
    public void clearItems() {
        SQLiteDatabase db = itemDatabaseOpenHelper.getWritableDatabase();
        itemDatabaseOpenHelper.clear(db);
        //create a new one
        itemDatabaseOpenHelper.onCreate(db);
    }

    //item database open helper
    private class ItemDatabaseOpenHelper extends SQLiteOpenHelper {

        private static final String DATABASE_NAME = "item_database.db";
        private static final String DATABASE_TABLE = "ItemTable";
        private static final int DATABASE_VERSION = 1;

        //SQL Statement to create a new database.
        private static final String DATABASE_CREATE = "create table "
                + DATABASE_TABLE + " (" + KEY_ID
                + " integer primary key autoincrement, " + KEY_NAME_COLUMN
                + " text not null, " + KEY_VALUE_COLUMN + " float, "
                + KEY_ACCESSIBLE_COLUMN + " integer);";

        public ItemDatabaseOpenHelper(Context context, String name,
                                      CursorFactory factory, int version) {
            super(context, name, factory, version);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL(DATABASE_CREATE);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            //drop the old table and create a new one
            clear(db);
            //create a new one
            onCreate(db);
        }

        public void clear(SQLiteDatabase db) {
            //drop the table
            db.execSQL("DROP TABLE " + DATABASE_TABLE);
        }
    }
}
